package fr.insa.clavardator.ui.chat;

import com.jfoenix.controls.JFXButton;
import fr.insa.clavardator.chat.FileMessage;
import fr.insa.clavardator.chat.Message;
import fr.insa.clavardator.users.CurrentUser;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.geometry.Pos;
import javafx.scene.control.Label;
import javafx.scene.layout.VBox;

import java.awt.*;
import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.text.DateFormat;
import java.util.ResourceBundle;

public class MessageListItemController implements Initializable {

	private Message currentMessage;

	@FXML
	private VBox container;
	@FXML
	private JFXButton button;
	@FXML
	private Label timestamp;

	/**
	 * Sets the message to display
	 *
	 * @param message The message to display
	 */
	public void setMessage(Message message) {
		if (!message.equals(currentMessage)) {
			currentMessage = message;
			String text = message.getText();
			if (message instanceof FileMessage) {
				FileMessage fileMessage = ((FileMessage) message);
				text += "\n<" + fileMessage.getFileName() + ">";
				button.setOnMouseClicked(event -> {
					Desktop desktop = Desktop.getDesktop();
					try {
						desktop.open(new File(fileMessage.getPath()));
					} catch (IOException e) {
						e.printStackTrace();
					}
				});
			}
			button.setText(text);
			timestamp.setText(DateFormat.getTimeInstance().format(message.getDate()));
			clearBackground();
			if (CurrentUser.getInstance().getId().equals(message.getSender().id)) {
				container.setAlignment(Pos.CENTER_RIGHT);
				button.getStyleClass().add("message-self");
			} else {
				container.setAlignment(Pos.CENTER_LEFT);
				button.getStyleClass().add("message-other");
			}
		}
	}

	/**
	 * Removes any style applied to the background
	 */
	private void clearBackground() {
		button.getStyleClass().remove("message-self");
		button.getStyleClass().remove("message-other");
	}

	@Override
	public void initialize(URL url, ResourceBundle rb) {}
}