package fr.insa.clavardator.users;

import fr.insa.clavardator.db.DatabaseController;
import fr.insa.clavardator.util.ErrorCallback;
import fr.insa.clavardator.util.Log;

import java.util.UUID;

public class CurrentUser extends User {
	private static CurrentUser instance = null;

	private State state;

	private CurrentUser() {
		super();
		state = State.UNINITIALIZED;
	}

	/**
	 * Returns the current.
	 */
	public static CurrentUser getInstance() {
		if (instance == null)
			instance = new CurrentUser();
		return instance;
	}

	private static String generateUniqueId() {
		return UUID.randomUUID().toString();
	}

	public void init(ErrorCallback errorCallback) {
		final DatabaseController db = new DatabaseController();
		db.getCurrentUser((user) -> {
			if (user == null) {
				id = generateUniqueId();
				Log.v(getClass().getSimpleName(), "No previous user found, generating id: " + id);
				db.addCurrentUser(
						new UserInformation(this),
						() -> setState(State.NONE),
						errorCallback);
			} else {
				id = user.id;
				if (user.getUsername() != null) {
					Log.v(getClass().getSimpleName(), "Last user found : " + id + " / " + getUsername());
					setUsername(user.getUsername());
				} else {
					Log.v(getClass().getSimpleName(), "No username found, asking user");
					setState(State.NONE);
				}
			}
		}, errorCallback);
	}

	@Override
	public void setUsername(String newUsername) {
		super.setUsername(newUsername);
		final DatabaseController db = new DatabaseController();
		db.updateCurrentUsername(new UserInformation(this),
				null,
				e -> Log.e(getClass().getSimpleName(), "Unable to update the username", e));
		Log.v(this.getClass().getSimpleName(),
				"Username changed to " + getUsername());
		setState(State.VALID);
	}

	public State getState() {
		return state;
	}

	public void setState(State state) {
		Log.v(this.getClass().getSimpleName(),
				"State changed from " + this.state.toString() + " to " + state.toString());
		instance.pcs.firePropertyChange("state", this.state, state);
		this.state = state;
	}

	public enum State {
		UNINITIALIZED,
		VALID,
		INVALID,
		NONE
	}

	public interface InitCallback {
		void onFinish();
	}
}
